# Our LivingThing class. It is the most 'generic', meaning
# both Heroes and Monsters are a type of LivingThing
class LivingThing
  # initialize tells the class how to create a new instance
  # of itself, i.e. how to create a new LivingThing.
  def initialize(name, health, power)
    @name = name      # @name means the Hero instance's name
    @health = health  # @ is used to make a variable an
    @power = power    # instance variable, so it's accessible
                      # *everywhere* inside this class. 
  end

  # A method that applies to all living things
  def dead?
    puts "I'm dead" if @health <= 0
    @health <= 0
  end
end

# Hero *inherits* from LivingThing, which means it gets
# all of LivingThing's properties and methods. In particular,
# Hero now has the initialize and dead? methods. However, 
# Hero still has the power to add additional methods, or
# override any of LivingThing's methods as it needs. 
class Hero < LivingThing
  # Since Hero has additional properties, it needs to 
  # *override* LivingThing's initialize method to achieve
  # its custom behavior
  def initialize(name, health, power, items)
    @name = name
    @health = health
    @power = power
    @items = items
    @exp = 0
  end

  # An attack method that's unique to Hero
  def attack(defendant)
    @name
    defendant[:health] -= @power
    @exp += 1
  end
end

# Monster also inherits from LivingThing, but behaves
# a bit differently than Hero. 
class Monster < LivingThing
  def initialize(name, health, power, type)
    @name = name
    @health = health
    @power = power
    @type = type
  end

  # An attack method that's unique to Monster
  def attack(defendant)
    defendant[:health] -= @power
  end
end

aaron = Hero.new("Aaron", 10, 2000, ["scruff", "squash racket"])

puts aaron.dead?